## sse-utils.sh -- script for updating files on OpenServer 5.0.5
##
## Available Subs:
## 
## GetSymlinkDestination(filename) - output destination of symlink, or
## 	null error.  
##	return 0: Success.
##	return 1: File is not a symlink.
## GetOrigOfFile(filename) - output the location of the original file
## 	under the .softmgmt tree for a given filename. 
##	return 0: Success.
##	return 1: filename isn't a symlink pointing to a regular file.
##	return 2: filename is a symlink, but doesn't point into /var/opt.
## PatchNewWithChanges(newfile,customfile) - diff and patch to update
##	newfile.  find differences between the original file and the
##	customized one, then patch those differences into newfile.  
##	return 0: Success, now copy $newfile to $customfile.
##	return 1: Patch failed. 
##	return 2: $customfile not a symlink to a regular file
##	return 3: $customfile identical to $newfile, no need to patch.
##	return 4: Failed, no .softmgmt file (shouldn't happen)
##

GetSymlinkDestination() {	# filename
	filename=$1
	if [ ! -h $filename ]; then
		return 1
	else
		/bin/ls -l $filename 2>/dev/null | awk '{print $NF}'
	fi
	return 0
}

GetOrigOfFile() {		# filename
	filename=$1
	if [ ! -f $filename ]; then
		return 1
	fi
	filename=`GetSymlinkDestination $filename` || return 1
	case $filename in
		/var/opt/*) ;;
		*) return 2 ;;
	esac
	echo $filename | awk -F/ '
          {
            # 1  2   3  4  5   6    7      8   9
            #  /var/opt/K/SCO/tcp/2.1.1Eb/etc/tcp
            for (i = 3; i <= NF; i++) {
              printf "/%s", $i
              if (i == 7) printf "/.softmgmt/var"
            }
            # -->  /opt/K/SCO/tcp/2.1.1Eb/.softmgmt/var/etc/tcp
          }'
	return 0
}

PatchNewWithChanges() {		# customfile newfile
	customfile=$1
	newfile=$2
	difffile=`basename $customfile`.diff
	cp -p $newfile $newfile.dist
	if origfile=`GetOrigOfFile $customfile`; then
		if [ -f $origfile ]; then
			diff -c $origfile $customfile > $difffile
			if [ ! -s $difffile ]; then
				return 3 # Use new, no change needed
			else
				if patch $newfile < $difffile; then
					return 0 # Patch succeeded, use new
				else
					return 1 # Failed, do manually
				fi
			fi
		else
			return 4 # Failed, no .softmgmt file (shouldn't happen)
		fi
	else
		return 2 # $customfile not a symlink to a regular file
	fi
}
